#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""

This script contains functions to perform the correction for sf6 sinks on sf6-derived mean AoA.
Called from sf6_correction_EXAMPLE.py

For details, please refer to accompanying publication:
    Garny et al, 2023, ACPD: Correction of stratospheric age-of-air derived from SF6 for the effect of chemical sinks

@author: Hella Garny, August 2023
"""

import numpy as np
from numpy.polynomial import polynomial as P
import numpy.ma as ma
from scipy import interpolate


def sf6_ft(SF6t):
    """ Function sf6_ft(sf6)
        This function provides the time-dependent function that incorporates the non-linear time evolution of SF6
        needed for the SF6 sink correction scheme.
        Input: annual mean SF6 time-series
        Returns function Ft(t) as annual time-series
    """
    ## correction factor F(t) = SF6(t-Gamma0)/ dSF6/dt(t-Gamma0)
    dSF6dt = np.zeros(SF6t.shape)
    Ft = np.zeros(SF6t.shape)
    Gamma0 = 4
    ny = 2
    for it in np.arange(ny,SF6t.shape[0]-ny,1):        
        dSF6dt[it] = (SF6t[it+ny]-SF6t[it-ny])/(ny*2)
        Ft[it] = SF6t[it]/dSF6dt[it]
    Ft[0:ny-1] = Ft[ny]
    Ft[SF6t.shape[0]-ny:SF6t.shape[0]-1] = Ft[SF6t.shape[0]-ny-2]
    # some smoothing
    for it in np.arange(ny,SF6t.shape[0]-ny,1):
        Ft[it] = np.mean(Ft[it-1:it+1])
    #shift back by Gamma0 years:
    Ft[Gamma0::] = Ft[0:SF6t.shape[0]-Gamma0]
    return Ft

def sf6_correct(aoa_sf6,Ft_time,alpha,beta):
    """ Function sf6_correct(aoa_sf6,Ft_time,alpha,beta):
        This function performs the correction of SF6-based AoA for chemical sinks,
        based on the function aoa_sf6 = aoa_ideal*(1+Ft(t)/(alpha*exp(beta*aoa_ideal)))        
        Input: SF6-based AoA at any location, Ft evaluated at measurement time t, and coefficents alpha and beta
        Returns corrected aoa value.
    """
    Gamma = np.arange(0,30,0.05)
    Gamma_tilde = Gamma * (1 +  Ft_time/(alpha*np.exp(beta*Gamma)))
    if (ma.isMaskedArray(aoa_sf6)):
        aoa_sf6.mask = False
        aoa_sf6 = aoa_sf6.data
    aoa_corr = np.interp(aoa_sf6,Gamma_tilde,Gamma)
    if (ma.isarray(aoa_sf6)):
        aoa_corr[aoa_sf6<0] = aoa_sf6[aoa_sf6<0]
    return aoa_corr



def sf6_correct_poly(aoa_sf6,Ft_time,coeff,valid_range):
    """ Function sf6_correct_poly(aoa_sf6,Ft_time,coeff,valid_range):
        This function performs the correction of SF6-based AoA for chemical sinks,
        based on the function aoa_sf6 = aoa_ideal*(1+Ft(t)/(tau_eff(aoa_ideal)), 
        where tau_eff is fitted as 'deg'-order polynomial with coefficients coeff
        Input: SF6-based AoA at any location, Ft evaluated at measurement time t, 
        fit coefficents of polynomial, and valid_range is a tuple of the range the polynomial fit function is valid for. 
        Outside this range, linear extrapolation will be used for the correction.
        Returns corrected aoa value.
    """
    Gamma = np.arange(valid_range[0],valid_range[1]+0.05,0.05)
    Gamma_tilde = Gamma * (1 +  Ft_time/(P.polyval(Gamma,coeff)))
    f = interpolate.interp1d(Gamma_tilde, Gamma, bounds_error=False,fill_value="extrapolate")
    if (ma.isMaskedArray(aoa_sf6)):
        aoa_sf6.mask = False
        aoa_sf6 = aoa_sf6.data
    aoa_corr =  f(aoa_sf6)
    if isinstance(aoa_sf6, np.ndarray):
        aoa_corr[aoa_sf6<0] = aoa_sf6[aoa_sf6<0]
    return aoa_corr




